@@ -21,48 +21,57 @@
 ##' @param method Method for proceeding numerical integration in the E-step. Default is pseudo-adaptive. 
 ##' @param opt Optimization method to fit a linear mixed effects model, either \code{nlminb} (default) or \code{optim}.
 ##' @return  Object of class \code{jmcs} with elements
-##'   \tabular{ll}{
-##'   \code{beta} \tab the vector of fixed effects for the linear mixed effects model. \cr
-##'   \code{gamma1} \tab the vector of fixed effects for type 1 failure for the survival model. \cr
-##'   \code{gamma2} \tab the vector of fixed effects for type 2 failure for the survival model. Valid only if \code{CompetingRisk = TRUE}. \cr
-##'   \code{nu1} \tab the vector of association parameter(s) for type 1 failure. \cr
-##'   \code{nu2} \tab the vector of association parameter(s) for type 2 failure. Valid only if \code{CompetingRisk = TRUE}. \cr
-##'   \code{H01} \tab the matrix that collects baseline hazards evaluated at each uncensored event time for type 1 failure. The first column denotes uncensored event times, 
-##'   the second column the number of events, and the third columns the hazards obtained by Breslow estimator. \cr
-##'   \code{H02} \tab the matrix that collects baseline hazards evaluated at each uncensored event time for type 2 failure. 
-##'   The data structure is the same as \code{H01}. Valid only if \code{CompetingRisk = TRUE}.  \cr
-##'   \code{Sig} \tab the variance-covariance matrix of the random effects. \cr
-##'   \code{sigma} \tab the variance of the measurement error for the linear mixed effects model. \cr
-##'   \code{iter} \tab the total number of iterations until convergence. \cr
-##'   \code{convergence} \tab convergence identifier: 1 corresponds to successful convergence, 
-##'   whereas 0 to a problem (i.e., when 0, usually more iterations are required). \cr
-##'   \code{vcov} \tab the variance-covariance matrix of all the fixed effects for both models. \cr
-##'   \code{sebeta} \tab the standard error of \code{beta}. \cr
-##'   \code{segamma1} \tab the standard error of \code{gamma1}. \cr
-##'   \code{segamma2} \tab the standard error of \code{gamma2}. Valid only if \code{CompetingRisk = TRUE}. \cr
-##'   \code{senu1} \tab the standard error of \code{nu1}. \cr
-##'   \code{senu2} \tab the standard error of \code{nu2}. Valid only if \code{CompetingRisk = TRUE}. \cr
-##'   \code{seSig} \tab the vector of standard errors of covariance of random effects. \cr
-##'   \code{sesigma} \tab the standard error of variance of measurement error for the linear mixed effects model. \cr
-##'   \code{loglike} \tab the log-likelihood value. \cr
-##'   \code{fitted} \tab a list with the fitted values. \cr
-##'   \code{fittedSurv} \tab the estimated survival rate evaluated at each uncensored event time. \cr
-##'   \code{FUNB} \tab the estimated random effects for each subject. \cr
-##'   \code{CompetingRisk} \tab logical value; TRUE if competing risks event are accounted for. \cr
-##'   \code{quadpoint} \tab the number of Gauss Hermite quadrature points used for numerical integration. \cr
-##'   \code{ydata} \tab the input longitudinal dataset for fitting a joint model. 
-##'   It has been re-ordered in accordance with descending observation times in \code{cdata}. \cr
-##'   \code{cdata} \tab the input survival dataset for fitting a joint model. 
-##'   It has been re-ordered in accordance with descending observation times. \cr
-##'   \code{PropEventType} \tab a frequency table of number of events. \cr
-##'   \code{LongitudinalSubmodel} \tab the component of the \code{long.formula}. \cr
-##'   \code{SurvivalSubmodel} \tab the component of the \code{surv.formula}. \cr
-##'   \code{random} \tab the component of the \code{random}. \cr
-##'   \code{call} \tab the matched call. \cr
-##'   \code{Quad.method} \tab the quadrature rule used for integration. 
-##'   If pseudo-adaptive quadrature rule is used, then return \code{pseudo-adaptive}. Otherwise return \code{standard}. \cr
-##'   \code{id} \tab the grouping vector for the longitudinal outcome. \cr
+##' \item{beta}{the vector of fixed effects for the linear mixed effects model.} 
+##' \item{gamma1}{the vector of fixed effects for type 1 failure for the survival model.}
+##' \item{gamma2}{the vector of fixed effects for type 2 failure for the survival model. 
+##' Valid only if \code{CompetingRisk = TRUE}.}
+##' \item{nu1}{the vector of association parameter(s) for type 1 failure.}
+##' \item{nu2}{the vector of association parameter(s) for type 2 failure. Valid only if \code{CompetingRisk = TRUE}.}
+##' \item{H01}{the matrix that collects baseline hazards evaluated at each uncensored event time for type 1 failure. 
+##' The first column denotes uncensored event times, the second column the number of events, and the third columns 
+##' the hazards obtained by Breslow estimator.}
+##' \item{H02}{the matrix that collects baseline hazards evaluated at each uncensored event time for type 2 failure. 
+##' The data structure is the same as \code{H01}. Valid only if \code{CompetingRisk = TRUE}.}
+##' \item{Sig}{the variance-covariance matrix of the random effects.}
+##' \item{sigma}{the variance of the measurement error for the linear mixed effects model.}
+##' \item{iter}{the total number of iterations until convergence.}
+##' \item{convergence}{convergence identifier: 1 corresponds to successful convergence, 
+##' whereas 0 to a problem (i.e., when 0, usually more iterations are required).}
+##' \item{vcov}{the variance-covariance matrix of all the fixed effects for both models.}
+##' \item{sebeta}{the standard error of \code{beta}.}
+##' \item{segamma1}{the standard error of \code{gamma1}.}
+##' \item{segamma2}{the standard error of \code{gamma2}. 
+##' Valid only if \code{CompetingRisk = TRUE}.}
+##' \item{senu1}{the standard error of \code{nu1}.}
+##' \item{senu2}{the standard error of \code{nu2}. Valid only if \code{CompetingRisk = TRUE}.}
+##' \item{seSig}{the vector of standard errors of covariance of random effects.}
+##' \item{sesigma}{the standard error of variance of measurement error for the linear mixed effects model.}
+##' \item{loglike}{the log-likelihood value.}
+##' \item{fitted}{a list with the fitted values:
+##'   \describe{
+##'   \item{resid}{the vector of estimated residuals for the linear mixed effects model.} 
+##'   \item{fitted}{the vector of fitted values for the linear mixed effects model.}
+##'   \item{fittedmar}{the vector of marginal fitted values for the linear mixed effects model.}
+##'   \item{residmar}{the vector of estimated marginal residuals for the linear mixed effects model.}
 ##'   }
+##' }
+##' \item{fittedSurv}{the estimated survival rate evaluated at each uncensored event time.}
+##' \item{FUNB}{the estimated random effects for each subject.}
+##' \item{CompetingRisk}{logical value; TRUE if a competing event are accounted for.}
+##' \item{quadpoint}{the number of Gauss Hermite quadrature points used for numerical integration.}
+##' \item{ydata}{the input longitudinal dataset for fitting a joint model.
+##' It has been re-ordered in accordance with descending observation times in \code{cdata}.}
+##' \item{cdata}{the input survival dataset for fitting a joint model.
+##' It has been re-ordered in accordance with descending observation times.}
+##' \item{PropEventType}{a frequency table of number of events.}
+##' \item{LongitudinalSubmodel}{the component of the \code{long.formula}.}
+##' \item{SurvivalSubmodel}{the component of the \code{surv.formula}.}
+##' \item{random}{the component of the \code{random}.}
+##' \item{call}{the matched call.}
+##' \item{Quad.method}{the quadrature rule used for integration. 
+##' If pseudo-adaptive quadrature rule is used, then return \code{pseudo-adaptive}. 
+##' Otherwise return \code{standard}.}
+##' \item{id}{the grouping vector for the longitudinal outcome.}
 ##' @author Shanpeng Li \email{lishanpeng0913@ucla.edu}
 ##' @seealso \code{\link{ranef}, \link{fixef}, \link{fitted.jmcs}, 
 ##' \link{residuals.jmcs}, \link{survfitjmcs}, \link{plot.jmcs}, \link{plot.survfitjmcs},
@@ -70,13 +79,16 @@
 ##' @examples 
 ##' 
 ##' require(FastJM)
+##' # Load a simulated longitudinal dataset
 ##' data(ydata)
+##' # Load a simulated survival dataset with two competing events
 ##' data(cdata)
 ##' \donttest{
+##' # Fit a joint model
 ##' fit <- jmcs(ydata = ydata, cdata = cdata, 
-##' long.formula = response ~ time + x1, 
-##' surv.formula = Surv(surv, failure_type) ~ x1 + x2, 
-##' random =  ~ time| ID)
+##'             long.formula = response ~ time + gender + x1 + race, 
+##'             surv.formula = Surv(surv, failure_type) ~ x1 + gender + x2 + race, 
+##'             random =  ~ time| ID)
 ##' fit
 ##' # Extract the parameter estimates of longitudinal sub-model fixed effects
 ##' fixef(fit, process = "Longitudinal")
@@ -87,6 +99,7 @@
 ##' # Obtain the variance-covariance matrix of all parameter estimates 
 ##' vcov(fit)
 ##' # Prediction of cumulative incidence for competing risks data
+##' # Predict the conditional probabilities for two patients who are alive (censored)
 ##' ND <- ydata[ydata$ID %in% c(419, 218), ]
 ##' ID <- unique(ND$ID)
 ##' NDc <- cdata[cdata$ID  %in% ID, ]
@@ -94,7 +107,8 @@
 ##'                        ynewdata = ND, 
 ##'                        cnewdata = NDc, 
 ##'                        u = seq(3, 4.8, by = 0.2), 
-##'                        M = 100)
+##'                        M = 100,
+##'                        seed = 100)
 ##' survfit
 ##' 
 ##' oldpar <- par(mfrow = c(2, 2))
@@ -122,8 +136,36 @@ jmcs <- function(ydata, cdata, long.formula, random = NULL, surv.formula, REML =
   if (!(method %in% c("standard", "pseudo-adaptive")))
     stop("Please choose one of the following metho for numerical integration: standard, pseudo-adaptive. See help() for details of the methods.")
   
+  if (!inherits(long.formula, "formula") || length(long.formula) != 3) {
+    stop("\nLinear mixed effects model must be a formula of the form \"resp ~ pred\"")
+  }
+  
+  if (!inherits(surv.formula, "formula") || length(surv.formula) != 3) {
+    stop("\nCox proportional hazards model must be a formula of the form \"Surv(.,.) ~ pred\"")
+  }
+  
+  long <- all.vars(long.formula)
+  survival <- all.vars(surv.formula)
   random.form <- all.vars(random)
   ID <- random.form[length(random.form)]
+  cnames <- colnames(cdata)
+  ynames <- colnames(ydata)
+  ##variable check
+  if (prod(long %in% ynames) == 0) {
+    Fakename <- which(long %in% ynames == FALSE)
+    stop(paste0("The variable ", long[Fakename], " not found"))
+  }
+  if (prod(survival %in% cnames) == 0) {
+    Fakename <- which(survival %in% cnames == FALSE)
+    stop(paste0("The variable ", survival[Fakename], " not found"))
+  }
+  if (!(ID %in% ynames)) {
+    stop(paste0("ID column ", ID, " not found in the longitudinal dataset!"))
+  }
+  if (!(ID %in% cnames)) {
+    stop(paste0("ID column ", ID, " not found in the survival dataset!"))
+  }
+  
   if (length(random.form) == 1) {
     RE <- NULL
     model <- "intercept"
@@ -131,6 +173,11 @@ jmcs <- function(ydata, cdata, long.formula, random = NULL, surv.formula, REML =
     RE <- random.form[-length(random.form)]
     model <- "interslope"
   }
+  
+  longfmla <- long.formula
+  survfmla <- surv.formula
+  rawydata <- ydata
+  rawcdata <- cdata
 
   getinit <- Getinit(cdata = cdata, ydata = ydata, long.formula = long.formula,
                      surv.formula = surv.formula,
@@ -138,8 +185,6 @@ jmcs <- function(ydata, cdata, long.formula, random = NULL, surv.formula, REML =
                      REML = REML, random = random, opt = opt)
   
   cdata <- getinit$cdata
-  ydata <- getinit$ydata
-
   
   survival <- all.vars(surv.formula)
   status <- as.vector(cdata[, survival[2]])
@@ -154,7 +199,9 @@ jmcs <- function(ydata, cdata, long.formula, random = NULL, surv.formula, REML =
     
     ## initialize parameters
     beta <- getinit$beta
+    namesbeta <- names(beta)
     gamma1 <- getinit$gamma1
+    namesgamma1 <- names(gamma1)
     gamma2 <- getinit$gamma2
     alpha1 <- getinit$alpha1
     alpha2 <- getinit$alpha2
@@ -171,7 +218,9 @@ jmcs <- function(ydata, cdata, long.formula, random = NULL, surv.formula, REML =
     
     ## initialize parameters
     beta <- getinit$beta
+    namesbeta <- names(beta)
     gamma1 <- getinit$gamma1
+    namesgamma1 <- names(gamma1)
     alpha1 <- getinit$alpha1
     Sig <- getinit$Sig
     p1a <- ncol(Sig)
@@ -310,8 +359,6 @@ jmcs <- function(ydata, cdata, long.formula, random = NULL, surv.formula, REML =
         print(sigma)
       }
       
-      
-      
       GetEfun <- GetE(beta, gamma1, gamma2, alpha1, alpha2, H01, H02, 
                       Sig, sigma, Z, X1, Y, X2, survtime, cmprsk, mdata, mdataS, xsmatrix, wsmatrix, method, Posbi, Poscov)
 
@@ -430,24 +477,19 @@ jmcs <- function(ydata, cdata, long.formula, random = NULL, surv.formula, REML =
       CH012 <- as.matrix(CH012)
       getfittedSurv <- getfittedSurv(gamma1, gamma2, X2, CH012, alpha1, alpha2, FUNB)
       
-      long <- all.vars(long.formula)
       survival <- all.vars(surv.formula)
       id <- ydata[, ID]
       
-      names(beta) <- c("(Intercept)", long[-1])
+      names(beta) <- namesbeta
       
-      names(gamma1) <- paste0(survival[-(1:2)], "_1")
-      names(gamma2) <- paste0(survival[-(1:2)], "_2")
+      names(gamma1) <- paste0(namesgamma1, "_1")
+      names(gamma2) <- paste0(namesgamma1, "_2")
       
       PropComp <- as.data.frame(table(cdata[, survival[2]]))
       
-      LongOut <- long[1]
-      LongX <- paste0(long[-1], collapse = "+")
-      FunCall_long <- as.formula(paste(LongOut, LongX, sep = "~"))
+      FunCall_long <- longfmla
       
-      SurvOut <- paste0("Surv(", survival[1], ",", survival[2], ")")
-      SurvX <- paste0(survival[-(1:2)], collapse = "+")
-      FunCall_survival <- as.formula(paste(SurvOut, SurvX, sep = "~"))
+      FunCall_survival <- survfmla
       
       ## return the joint modelling result
       mycall <- match.call()
@@ -456,7 +498,7 @@ jmcs <- function(ydata, cdata, long.formula, random = NULL, surv.formula, REML =
                      H02, Sig, sigma, iter, convergence, vcov, sebeta, segamma1,
                      segamma2, sealpha1, sealpha2, seSig, sesigma, getloglike, 
                      getfitted, getfittedSurv, FUNB, CompetingRisk,
-                     quadpoint, ydata, cdata, PropComp, FunCall_long,
+                     quadpoint, rawydata, rawcdata, PropComp, FunCall_long,
                      FunCall_survival, random, mycall, method, id)
       
       names(result) <- c("beta", "gamma1", "gamma2", "nu1", "nu2", "H01", "H02", "Sig", 
@@ -586,23 +628,24 @@ jmcs <- function(ydata, cdata, long.formula, random = NULL, surv.formula, REML =
       CH01 <- as.matrix(CH01)
       getfittedSurv <- getfittedSurvSF(gamma1, X2, CH01, alpha1, FUNB)
       
-      long <- all.vars(long.formula)
       survival <- all.vars(surv.formula)
       id <- ydata[, ID]
       
-      names(beta) <- c("(Intercept)", long[-1])
-      
-      names(gamma1) <- paste0(survival[-(1:2)], "_1")
+      names(beta) <- namesbeta 
+      names(gamma1) <- paste0(namesgamma1, "_1")
+      # names(gamma1) <- paste0(survival[-(1:2)], "_1")
       
       PropComp <- as.data.frame(table(cdata[, survival[2]]))
       
-      LongOut <- long[1]
-      LongX <- paste0(long[-1], collapse = "+")
-      FunCall_long <- as.formula(paste(LongOut, LongX, sep = "~"))
+      survival <- all.vars(survfmla)
       
-      SurvOut <- paste0("Surv(", survival[1], ",", survival[2], ")")
-      SurvX <- paste0(survival[-(1:2)], collapse = "+")
-      FunCall_survival <- as.formula(paste(SurvOut, SurvX, sep = "~"))
+      FunCall_long <- longfmla
+      
+      # SurvOut <- paste0("Surv(", survival[1], ",", survival[2], ")")
+      # SurvX <- paste0(survival[-(1:2)], collapse = "+")
+      # FunCall_survival <- as.formula(paste(SurvOut, SurvX, sep = "~"))
+      # 
+      FunCall_survival <- survfmla
       
       ## return the joint modelling result
       mycall <- match.call()
@@ -610,7 +653,7 @@ jmcs <- function(ydata, cdata, long.formula, random = NULL, surv.formula, REML =
       result <- list(beta, gamma1, alpha1, H01, Sig, sigma, iter, convergence, 
                      vcov, sebeta, segamma1, sealpha1, seSig, sesigma, getloglike, 
                      getfitted, getfittedSurv, FUNB, CompetingRisk,
-                     quadpoint, ydata, cdata, PropComp, FunCall_long, FunCall_survival, 
+                     quadpoint, rawydata, rawcdata, PropComp, FunCall_long, FunCall_survival, 
                      random, mycall, method, id)
       
       names(result) <- c("beta", "gamma1", "nu1", "H01", "Sig", "sigma",
